/*----------------------------------------------------------------------------------------------------------------------------------------
 * stm32flash.h - flash STM32 per Bootloader API
 *
 * Copyright (c) 2017 Frank Meyer - frank(at)fli4l.de
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *----------------------------------------------------------------------------------------------------------------------------------------
 */

#include <ESP8266WiFi.h>
#include <ESP8266httpUpdate.h>
#include <FS.h>
#include "base.h"
#include "http.h"
#include "httpclient.h"

#define STM32_BOOT0_PIN             16
#define STM32_RESET_PIN             14

#define STM32_BEGIN                 0x7F
#define STM32_ACK                   0x79
#define STM32_NACK                  0x1F

#define STM32_CMD_GET               0x00    // gets the version and the allowed commands supported by the current version of the bootloader
#define STM32_CMD_GET_VERSION       0x01    // gets the bootloader version and the Read Protection status of the Flash memory
#define STM32_CMD_GET_ID            0x02    // gets the chip ID
#define STM32_CMD_READ_MEMORY       0x11    // reads up to 256 bytes of memory starting from an address specified by the application
#define STM32_CMD_GO                0x21    // jumps to user application code located in the internal Flash memory or in SRAM
#define STM32_CMD_WRITE_MEMORY      0x31    // writes up to 256 bytes to the RAM or Flash memory starting from an address specified by the application
#define STM32_CMD_ERASE             0x43    // erases from one to all the Flash memory pages
#define STM32_CMD_EXT_ERASE         0x44    // erases from one to all the Flash memory pages using two byte addressing mode (available only for v3.0 usart bootloader versions and above).
#define STM32_CMD_WRITE_PROTECT     0x63    // enables the write protection for some sectors
#define STM32_CMD_WRITE_UNPROTECT   0x73    // disables the write protection for all Flash memory sectors
#define STM32_CMD_READOUT_PROTECT   0x82    // enables the read protection
#define STM32_CMD_READOUT_UNPROTECT 0x92    // disables the read protection

#define wait_for_ack(t)                                 \
do                                                      \
{                                                       \
    int x;                                              \
    if ((x = stm32_serial_poll(t, 1)) < 0)              \
    {                                                   \
        http_send ("timeout, no ACK<BR>\r\n");          \
        return -1;                                      \
    }                                                   \
    else if (x != STM32_ACK)                            \
    {                                                   \
        char buf[32];                                   \
                                                        \
        http_send ("no ACK: ");                         \
        sprintf (buf, "(0x%02x)<BR>\r\n", x);           \
        http_send (buf);                                \
        return -1;                                      \
    }                                                   \
} while (0)

#define poll_byte(n,t)                                  \
do                                                      \
{                                                       \
    if ((n = stm32_serial_poll (t, 1)) < 0)             \
    {                                                   \
        http_send ("timeout, no ACK<BR>\r\n");          \
        return -1;                                      \
    }                                                   \
} while (0)

/*-------------------------------------------------------------------------------------------------------------------------------------------
 * poll a character from serial
 *-------------------------------------------------------------------------------------------------------------------------------------------
 */
int
stm32_serial_poll (long timeout, int log_error)
{
    long    curtime = millis ();
    unsigned int ch;

    Serial.flush ();

    while (Serial.available() <= 0)
    {
        yield ();

        if (millis () - curtime >= timeout)
        {
            if (log_error)
            {
                http_send ("Timeout<BR>\r\n");
            }
            return -1;
        }
    }

    ch = Serial.read ();
    return (int) ch;
}

#define STM32_BUFLEN    256
static uint8_t stm32_buf[STM32_BUFLEN + 1];                        // one more byte for checksum

/*-------------------------------------------------------------------------------------------------------------------------------------------
 * write a bootloader command to STM32
 *-------------------------------------------------------------------------------------------------------------------------------------------
 */
static void
stm32_write_cmd (uint8_t cmd)
{
    stm32_buf[0] = cmd;
    stm32_buf[1] = ~cmd;
    Serial.write (stm32_buf, 2);
    Serial.flush ();
}

/*-------------------------------------------------------------------------------------------------------------------------------------------
 * STM32 bootloader command: GET
 *-------------------------------------------------------------------------------------------------------------------------------------------
 */
static int
stm32_get (void)
{
    int i;
    int ch;
    int n_bytes;

    stm32_write_cmd (STM32_CMD_GET);
    wait_for_ack (200);

    poll_byte(n_bytes, 200);
    n_bytes++;

    for (i = 0; i < n_bytes; i++)
    {
        poll_byte(ch, 200);

        if (i < STM32_BUFLEN)
        {
            stm32_buf[i] = ch;
        }
    }

    wait_for_ack (200);
    return n_bytes;
}

/*-------------------------------------------------------------------------------------------------------------------------------------------
 * STM32 bootloader command: GET VERSION
 *-------------------------------------------------------------------------------------------------------------------------------------------
 */
static int
stm32_get_version (void)
{
    int i;
    int ch;
    int n_bytes;

    stm32_write_cmd (STM32_CMD_GET_VERSION);
    wait_for_ack (200);

    n_bytes = 3;

    for (i = 0; i < n_bytes; i++)
    {
        poll_byte(ch, 200);

        if (i < STM32_BUFLEN)
        {
            stm32_buf[i] = ch;
        }
    }

    wait_for_ack (200);
    return n_bytes;
}

/*-------------------------------------------------------------------------------------------------------------------------------------------
 * STM32 bootloader command: GET ID
 *-------------------------------------------------------------------------------------------------------------------------------------------
 */
static int
stm32_get_id (void)
{
    int i;
    int ch;
    int n_bytes;

    stm32_write_cmd (STM32_CMD_GET_ID);
    wait_for_ack (200);

    poll_byte(n_bytes, 200);
    n_bytes++;

    for (i = 0; i < n_bytes; i++)
    {
        poll_byte(ch, 200);

        if (i < STM32_BUFLEN)
        {
            stm32_buf[i] = ch;
        }
    }

    wait_for_ack (200);
    return n_bytes;
}

/*-------------------------------------------------------------------------------------------------------------------------------------------
 * STM32 bootloader command: READ MEMORY
 *-------------------------------------------------------------------------------------------------------------------------------------------
 */
static int
stm32_read_memory (uint32_t address, unsigned int len)
{
    int i;
    int ch;
    int n_bytes;

    stm32_write_cmd (STM32_CMD_READ_MEMORY);
    wait_for_ack (200);

    stm32_buf[0] = (address >> 24) & 0xFF;
    stm32_buf[1] = (address >> 16) & 0xFF;
    stm32_buf[2] = (address >>  8) & 0xFF;
    stm32_buf[3] = address & 0xFF;
    stm32_buf[4] = stm32_buf[0] ^ stm32_buf[1] ^ stm32_buf[2] ^ stm32_buf[3];

    Serial.write (stm32_buf, 5);
    Serial.flush ();
    wait_for_ack (200);

    stm32_buf[0] = len - 1;
    stm32_buf[1] = ~stm32_buf[0];
    Serial.write (stm32_buf, 2);
    Serial.flush ();
    wait_for_ack (200);

    for (i = 0; i < len; i++)
    {
        poll_byte(ch, 200);

        if (i < STM32_BUFLEN)
        {
            stm32_buf[i] = ch;
        }
    }

    // wait_for_ack (200);                        // no ACK
    return len;
}

/*-------------------------------------------------------------------------------------------------------------------------------------------
 * STM32 bootloader command: GO
 *-------------------------------------------------------------------------------------------------------------------------------------------
 */
static int
stm32_go (uint32_t address)
{
    int ch;
    int n_bytes;

    stm32_write_cmd (STM32_CMD_GO);
    wait_for_ack (200);

    stm32_buf[0] = (address >> 24) & 0xFF;
    stm32_buf[1] = (address >> 16) & 0xFF;
    stm32_buf[2] = (address >>  8) & 0xFF;
    stm32_buf[3] = address & 0xFF;
    stm32_buf[4] = stm32_buf[0] ^ stm32_buf[1] ^ stm32_buf[2] ^ stm32_buf[3];

    Serial.write (stm32_buf, 5);
    Serial.flush ();
    wait_for_ack (200);
    return 0;
}

/*-------------------------------------------------------------------------------------------------------------------------------------------
 * STM32 bootloader command: WRITE MEMORY
 *
 * len must be a multiple of 4!
 *-------------------------------------------------------------------------------------------------------------------------------------------
 */
static int
stm32_write_memory (uint8_t * image, uint32_t address, unsigned int len)
{
    uint8_t sum;
    int     i;

    if (len > 256)
    {
        return -1;
    }

    stm32_write_cmd (STM32_CMD_WRITE_MEMORY);
    wait_for_ack (200);

    stm32_buf[0] = (address >> 24) & 0xFF;
    stm32_buf[1] = (address >> 16) & 0xFF;
    stm32_buf[2] = (address >>  8) & 0xFF;
    stm32_buf[3] = address & 0xFF;
    stm32_buf[4] = stm32_buf[0] ^ stm32_buf[1] ^ stm32_buf[2] ^ stm32_buf[3];

    Serial.write (stm32_buf, 5);
    Serial.flush ();
    wait_for_ack (200);

    stm32_buf[0] = len - 1;
    sum = stm32_buf[0];
    Serial.write (stm32_buf, 1);
    Serial.flush ();

    for (i = 0; i < len; i++)
    {
        sum ^= image[i];
    }

#if 1
    Serial.write (image, len);
#else
    for (i = 0; i < len; i++)
    {
        Serial.write (image + i, 1);
        Serial.flush ();
        delay (50);
    }
#endif
    stm32_buf[0] = sum;
    Serial.write (stm32_buf, 1);
    Serial.flush ();
    wait_for_ack (200);
    return 0;
}

/*-------------------------------------------------------------------------------------------------------------------------------------------
 * STM32 bootloader command: WRITE UNPROTECT
 *-------------------------------------------------------------------------------------------------------------------------------------------
 */
static int
stm32_write_unprotect (void)
{
    int ch;
    int n_bytes;

    stm32_write_cmd (STM32_CMD_WRITE_UNPROTECT);
    wait_for_ack (200);         // 2x
    wait_for_ack (200);
    return 0;
}

/*-------------------------------------------------------------------------------------------------------------------------------------------
 * STM32 bootloader command: ERASE
 *
 * n_pages == 0: global erase
 * 1 <= n_pages <= 256: erase N pages
 *-------------------------------------------------------------------------------------------------------------------------------------------
 */
static int
stm32_erase (uint8_t * pagenumbers, uint16_t n_pages)
{
    int     i;
    int     ch;
    int     n_bytes;
    uint8_t sum;

    if (n_pages > 256)
    {
        return -1;
    }

    stm32_write_cmd (STM32_CMD_ERASE);
    wait_for_ack (200);

    if (n_pages == 0)
    {
        stm32_buf[0] = 0xFF;
        stm32_buf[1] = ~stm32_buf[0];

        Serial.write (stm32_buf, 2);
        Serial.flush ();
        wait_for_ack (10000);
    }
    else
    {
        stm32_buf[0] = n_pages - 1;
        stm32_buf[1] = ~stm32_buf[0];
        Serial.write (stm32_buf, 2);
        Serial.flush ();
        wait_for_ack (200);

        sum = 0;

        for (i = 0; i < n_pages; i++)
        {
            sum ^= pagenumbers[i];
        }

        Serial.write (pagenumbers, n_pages);
        stm32_buf[0] = sum;
        Serial.write (stm32_buf, 1);
        Serial.flush ();
        wait_for_ack (10000);                                    // wait 10 sec
    }

    return 0;
}

#define PAGESIZE        256
static uint32_t         start_address   = 0x00000000;                   // address of program start

static int
stm32_flash_image ()
{
    char            linebuf[256];
    char            logbuf[128];
    //uint8_t         flash_image[256];
    int             i;
    //int             ch;
    int             len;
    int             line;
    uint8_t         pagebuf[PAGESIZE];
    uint32_t        pageaddr        = 0xffffffff;
    uint32_t        pageidx         = 0;
    unsigned char   datalen;
    uint32_t        address_min     = 0xffffffff;               // minimum address (incl.)
    uint32_t        address_max     = 0x00000000;               // maximum address (incl.)
#if 0
    uint32_t        last_address    = 0x00000000;
#endif
    uint32_t        drlo;                                       // DATA Record Load Offset (current address, 2 bytes)
    uint32_t        ulba            = 0x00000000;               // Upper Linear Base Address (address offset, 4 bytes)
    unsigned char   dri;                                        // Data Record Index
    uint32_t        bytes_written;
    int             cnt;
    int             errors = 0;
    int             rtc = 0;

    http_send ("Flashing STM32...<br/>");

	File f = SPIFFS.open("stm32.hex", "r");
	http_send((String("File size: ") + itoa(f.size(), linebuf, 10)).c_str());
	http_send("<br/>");

    line = 1;
    bytes_written = 0;

    while(f.available())
    {
		cnt = f.readBytesUntil ('\n', linebuf, 256);
        linebuf[cnt] = '\0';
        {
            len = strlen (linebuf);
    
            if (linebuf[0] == ':' && len >= 11)
            {
                unsigned char   addrh;
                unsigned char   addrl;
                unsigned char   rectype;
                char *          dataptr;
                unsigned char   chcksum;
                int             sum = 0;
                unsigned char   ch;
    
                datalen = hex2toi (linebuf +  1);
                addrh   = hex2toi (linebuf +  3);
                addrl   = hex2toi (linebuf +  5);
                drlo    = (addrh << 8 | addrl);
                rectype = hex2toi (linebuf + 7);
                dataptr = linebuf + 9;
    
                sum = datalen + addrh + addrl + rectype;
    
                if (len == 9 + 2 * datalen + 2)
                {
                    if (rectype == 0)                               // Data Record
                    {
                        for (dri = 0; dri < datalen; dri++)
                        {
                            ch = hex2toi (dataptr);
    
                            pageidx = ulba + drlo + dri;
    
                            if (pageidx - pageaddr >= PAGESIZE)
                            {
                                if (pageaddr != 0xffffffff)
                                {
                                    // flash (pagebuf, pageaddr, pageidx - pageaddr);
                                    if (stm32_write_memory (pagebuf, pageaddr, pageidx - pageaddr) < 0)
                                    {
                                        rtc = -1;
                                        break;
                                    }

                                    yield ();

                                    if (stm32_read_memory (pageaddr, pageidx - pageaddr) < 0)
                                    {
                                        rtc = -1;
                                        break;
                                    }

                                    yield ();

                                    if (memcmp (pagebuf, stm32_buf, pageidx - pageaddr) != 0)
                                    {
                                        errors++;
                                        char tmpbuf[32];
                                        http_send ("verify failed at address=\r\n");
                                        sprintf (tmpbuf, "%08X \r\n", pageaddr);
                                        http_send (tmpbuf);
                                        sprintf (tmpbuf, "len=%d<BR>\r\n", pageidx - pageaddr);
                                        http_send (tmpbuf);
                                        http_send ("pagebuf:<BR><pre>\r\n");

                                        for (i = 0; i < pageidx - pageaddr; i++)
                                        {
                                            sprintf (tmpbuf, "%02X ", pagebuf[i]);
                                            http_send (tmpbuf);
                                            yield ();
                                        }

                                        http_send ("</pre><BR>\r\n");

                                        http_send ("stm32_buf:<BR><pre>\r\n");

                                        for (i = 0; i < pageidx - pageaddr; i++)
                                        {
                                            sprintf (tmpbuf, "%02X ", stm32_buf[i]);
                                            http_send (tmpbuf);
                                            yield ();
                                        }

                                        http_send ("</pre><BR>\r\n");
                                        rtc = -1; 
                                        break;
                                    }

                                    bytes_written += pageidx - pageaddr;
                                }
    
                                pageaddr = ulba + drlo + dri;
                                memset (pagebuf, 0xFF, PAGESIZE);
                            }
    
                            pagebuf[pageidx - pageaddr] = ch;
#if 0
                            if (last_address && last_address + 1 != ulba + drlo + dri)
                            {
                                sprintf (logbuf, "Sprung Zeile %d\r\n", line);
                                http_send (logbuf);
                            }
                            last_address = ulba + drlo + dri;
#endif
    
                            if (address_min > ulba + drlo + dri)
                            {
                                address_min = ulba + drlo + dri;
                            }
    
                            if (address_max < ulba + drlo + dri)
                            {
                                address_max = ulba + drlo + dri;
                            }
    
                            sum += ch;
                            dataptr +=2;
                        }

                        if (rtc == -1)
                        {
                            break;
                        }
                    }
                    else if (rectype == 1)                          // End of File Record
                    {
                        for (dri = 0; dri < datalen; dri++)
                        {
                            ch = hex2toi (dataptr);
    
                            sum += ch;
                            dataptr +=2;
                        }
    
                        if (pageidx - pageaddr > 0)
                        {
                            pageidx++;                              // we have to go behind last byte
                            // flash (pagebuf, pageaddr, pageidx - pageaddr);
                            if (stm32_write_memory (pagebuf, pageaddr, pageidx - pageaddr) < 0)
                            {
                                rtc = -1;
                                break;
                            }

                            yield ();

                            if (stm32_read_memory (pageaddr, pageidx - pageaddr) < 0)
                            {
                                rtc = -1;
                                break;
                            }

                            yield ();

                            if (memcmp (pagebuf, stm32_buf, pageidx - pageaddr) != 0)
                            {
                                errors++;

                                char tmpbuf[32];

                                http_send ("verify failed at address \r\n");
                                sprintf (tmpbuf, "%08X<BR>\r\n", pageaddr);
                                http_send (tmpbuf);
                                http_send ("pagebuf:<BR><pre>\r\n");

                                for (i = 0; i < pageidx - pageaddr; i++)
                                {
                                    sprintf (tmpbuf, "%02X ", pagebuf[i]);
                                    http_send (tmpbuf);
                                    yield ();
                                }

                                http_send ("</pre><BR>\r\n");

                                http_send ("stm32_buf:<BR><pre>\r\n");

                                for (i = 0; i < pageidx - pageaddr; i++)
                                {
                                    sprintf (tmpbuf, "%02X ", stm32_buf[i]);
                                    http_send (tmpbuf);
                                    yield ();
                                }

                                http_send ("</pre><BR>\r\n");
                                rtc = -1; 
                                break;
                            }

                            bytes_written += pageidx - pageaddr;
                            pageidx = 0;
                            memset (pagebuf, 0xFF, PAGESIZE);
                        }
                        break;                                      // stop reading here
                    }
                    else if (rectype == 4)                          // Extended Linear Address Record
                    {
                        if (drlo == 0)
                        {
                            ulba = 0;
    
                            for (dri = 0; dri < datalen; dri++)
                            {
                                ch = hex2toi (dataptr);
    
                                ulba <<= 8;
                                ulba |= ch;
    
                                sum += ch;
                                dataptr +=2;
                            }
    
                            ulba <<= 16;
                        }
                        else
                        {
                            sprintf (logbuf, "line %d: rectype = %d: address field is 0x%04X\r\n", line, rectype, drlo);
                            http_send (logbuf);
                            rtc = -1;
                            break;
                        }
                    }
                    else if (rectype == 5)                          // Start Linear Address Record
                    {
                        if (drlo == 0)
                        {
                            start_address = 0;
    
                            for (dri = 0; dri < datalen; dri++)
                            {
                                ch = hex2toi (dataptr);
    
                                start_address <<= 8;
                                start_address |= ch;
    
                                sum += ch;
                                dataptr +=2;
                            }
                        }
                        else
                        {
                            sprintf (logbuf, "line %d: rectype = %d: address field is 0x%04X\r\n", line, rectype, drlo);
                            http_send (logbuf);
                            rtc = -1;
                            break;
                        }
                    }
                    else
                    {
                        sprintf (logbuf, "line %d: unsupported record type: %d\r\n", line, rectype);
                        http_send (logbuf);
                        rtc = -1;
                        break;
                    }
    
                    sum = (0x100 - (sum & 0xff)) & 0xff;
                    chcksum = hex2toi (dataptr);
    
                    if (sum != chcksum)
                    {
                        sprintf (logbuf, "invalid checksum: sum: 0x%02X chcksum: 0x%02X\r\n", sum, chcksum);
                        http_send (logbuf);
                        rtc = -1;
                        break;
                    }
                }
                else
                {
                    sprintf (logbuf, "invalid len: %d, expected %d\r\n", len, 12 + 2 * datalen + 2);
                    http_send (logbuf);
                    rtc = -1;
                    break;
                }
            }
            else
            {
                sprintf (logbuf, "invalid INTEL HEX format, len: %d\r\n", len);
                http_send (logbuf);
                rtc = -1;
                break;
            }
        }
    }
	f.close();

    httpclient_stop ();

    sprintf (logbuf, "Bytes flashed: %ld<BR>\r\n", bytes_written);
    http_send (logbuf);

    sprintf (logbuf, "errors: %d<BR>\r\n", errors);
    http_send (logbuf);

    if (rtc == 0)
    {
        http_send ("Flash successful<BR>\r\n");
    }
    else
    {
        http_send ("Flash failed<BR>\r\n");
    }

    return rtc;
}

/*-------------------------------------------------------------------------------------------------------------------------------------------
 * start STM32 bootloader
 *-------------------------------------------------------------------------------------------------------------------------------------------
 */
static int
stm32_bootloader (int do_unprotect)
{
    char        buffer[256];
    //uint8_t     pagenumbers[1] = { 0 };
    int         i;
    int         ch;
    int         rtc = 0;

    buffer[0] = STM32_BEGIN;

    for (i = 0; i < 10; i++)
    {
        http_send ("Trying...<BR>\r\n");
        Serial.write (buffer, 1);
        Serial.flush ();
        ch = stm32_serial_poll (200, 1);
 
        if (ch == STM32_ACK)
        {
            break;
        }
    }

    if (i < 10)
    {
        if (do_unprotect)
        {
            rtc = stm32_write_unprotect ();

            if (rtc == 0)
            {
                delay (500);

                http_send ("unprotected<BR>\r\n");

                buffer[0] = STM32_BEGIN;
                for (i = 0; i < 10; i++)
                {
                    http_send ("Trying again...<BR>\r\n");
                    Serial.write (buffer, 1);
                    Serial.flush ();
                    ch = stm32_serial_poll (200, 1);
             
                    if (ch == STM32_ACK)
                    {
                        break;
                    }
                }
            }
        }
        else
        {
            rtc = 0;
        }

        if (rtc == 0)
        {
            http_send ("Erasing flash...<BR>\r\n");
            rtc = stm32_erase (0, 0);
        }

        if (rtc == 0)
        {
            http_send ("Flashing Image...<BR>\r\n");
            rtc = stm32_flash_image ();
        }
    }
    
    return rtc;
}

/*-------------------------------------------------------------------------------------------------------------------------------------------
 * download image
 * cmd = 0xFF: run some tests
 *-------------------------------------------------------------------------------------------------------------------------------------------
 */
bool
stm32_flash_download_image(const char * host, const char * path)
{
    http_send ("Downloading the image file... <br/>");
    http_flush();

    if (httpclient (host, path) < 0)
    {
        http_send ("<font color='red'>host connection failed</font><BR>");
        return false;
    }
    String line;
	File f = SPIFFS.open("stm32.hex", "w");

    while(httpclient_read_line(line, 256))
    {
    	//Serial.println(line);
		f.write((const unsigned char*)line.c_str(), line.length());
		f.write('\n');
    }
	f.close();
	return true;
}
/*-------------------------------------------------------------------------------------------------------------------------------------------
 * flash STM32
 * cmd = 0xFF: run some tests
 *-------------------------------------------------------------------------------------------------------------------------------------------
 */
void
stm32_flash_from_server (const char * host, const char * path, int do_unprotect)
{
	SPIFFS.begin();
	stm32_flash_download_image(host, path);

    digitalWrite(STM32_BOOT0_PIN, HIGH);                    // activate BOOT0
    pinMode(STM32_RESET_PIN, OUTPUT);                       // RESET to output
    digitalWrite(STM32_RESET_PIN, LOW);                     // activate RESET
    Serial.flush ();
    delay (200);
    Serial.end ();                                          // end old serial
    Serial.begin (115200, SERIAL_8E1);                      // switch to 8 bit even parity
    Serial.swap ();                                         // swap UART pins: GPIO15 is now TX, GPIO13 is now RX
    delay (200);                                            // wait 200ms
    pinMode(STM32_RESET_PIN, INPUT);                        // release RESET

    while (stm32_serial_poll (1000, 0) >= 0)                // flush characters in serial input
    {
        ;
    }

    http_send ("Start Bootloader<BR>\r\n");
    stm32_bootloader (do_unprotect);
    http_send ("End Bootloader<BR>\r\n");

    Serial.end ();                                          // end old serial
    Serial.begin (115200, SERIAL_8N1);                      // switch to 8 bit no parity
	SPIFFS.remove("/stm32.hex");
	SPIFFS.end();
}

/*-------------------------------------------------------------------------------------------------------------------------------------------
 * flash STM32
 * cmd = 0xFF: run some tests
 *-------------------------------------------------------------------------------------------------------------------------------------------
 */
void
stm32_flash_from_local (const String& path)
{
    digitalWrite(STM32_BOOT0_PIN, HIGH);                    // activate BOOT0
    pinMode(STM32_RESET_PIN, OUTPUT);                       // RESET to output
    digitalWrite(STM32_RESET_PIN, LOW);                     // activate RESET
    Serial.flush ();
    delay (200);
    Serial.end ();                                          // end old serial
    Serial.begin (115200, SERIAL_8E1);                      // switch to 8 bit even parity
    Serial.swap ();                                         // swap UART pins: GPIO15 is now TX, GPIO13 is now RX
    delay (200);                                            // wait 200ms
    pinMode(STM32_RESET_PIN, INPUT);                        // release RESET

    while (stm32_serial_poll (1000, 0) >= 0)                // flush characters in serial input
    {
        ;
    }

    http_send ("Start Bootloader<BR>\r\n");
    stm32_bootloader (1);
    http_send ("End Bootloader<BR>\r\n");

    Serial.end ();                                          // end old serial
    Serial.begin (115200, SERIAL_8N1);                      // switch to 8 bit no parity
}

void
stm32_reset (void)
{
    digitalWrite(STM32_BOOT0_PIN, LOW);                     // deactivate BOOT0
    pinMode(STM32_RESET_PIN, OUTPUT);                       // RESET to output
    digitalWrite(STM32_RESET_PIN, LOW);                     // activate RESET
    delay (200);                                            // wait 200ms
    pinMode(STM32_RESET_PIN, INPUT);                        // release RESET
}

void
stm32_flash_init (void)
{
    pinMode(STM32_BOOT0_PIN, OUTPUT);
    digitalWrite(STM32_BOOT0_PIN, LOW);
    pinMode(STM32_RESET_PIN, INPUT);
    pinMode(13, INPUT);     // swapped RX, default is input
    pinMode(15, INPUT);     // swapped TX, default is input
}
